package com.sun.xml.internal.ws.api.server;

import com.sun.xml.internal.ws.util.Pool;
import com.oracle.webservices.internal.api.message.PropertySet;
import com.sun.istack.internal.Nullable;
import java.io.IOException;
import com.sun.xml.internal.ws.api.message.Packet;
import com.sun.istack.internal.NotNull;
import com.sun.xml.internal.ws.api.pipe.Codec;

public abstract class AbstractServerAsyncTransport<T>
{
    private final WSEndpoint endpoint;
    private final AbstractServerAsyncTransport.CodecPool codecPool;
    
    public AbstractServerAsyncTransport(WSEndpoint endpoint) {
        this.endpoint = endpoint;
        this.codecPool = new CodecPool(endpoint);
    }
    
    protected Packet decodePacket(T connection, @NotNull Codec codec) throws IOException {
        Packet packet = new Packet();
        packet.acceptableMimeTypes = this.getAcceptableMimeTypes(connection);
        packet.addSatellite(this.getPropertySet(connection));
        packet.transportBackChannel = this.getTransportBackChannel(connection);
        return packet;
    }
    
    protected abstract void encodePacket(T p0, @NotNull Packet p1, @NotNull Codec p2) throws IOException;
    
    @Nullable
    protected abstract String getAcceptableMimeTypes(T p0);
    
    @Nullable
    protected abstract TransportBackChannel getTransportBackChannel(T p0);
    
    @NotNull
    protected abstract PropertySet getPropertySet(T p0);
    
    @NotNull
    protected abstract WebServiceContextDelegate getWebServiceContextDelegate(T p0);
    
    protected void handle(final T connection) throws IOException {
        final Codec codec = (Codec)this.codecPool.take();
        Packet request = this.decodePacket(connection, codec);
        if (!request.getMessage().isFault()) {
            this.endpoint.schedule(request, new WSEndpoint.CompletionCallback() {
                @Override
                public void onCompletion(@NotNull Packet response) {
                    try {
                        AbstractServerAsyncTransport.this.encodePacket(connection, response, codec);
                    }
                    catch (IOException ioe) {
                        ioe.printStackTrace();
                    }
                    AbstractServerAsyncTransport.this.codecPool.recycle(codec);
                }
            });
        }
    }
    
    private static final class CodecPool extends Pool<Codec>
    {
        WSEndpoint endpoint;
        
        CodecPool(WSEndpoint endpoint) {
            this.endpoint = endpoint;
        }
        
        protected Codec create() {
            return this.endpoint.createCodec();
        }
    }
}
